@@ -1,11 +1,13 @@
 import logging
 import traceback
+from ast import Or
 from datetime import datetime, timedelta
+from decimal import Decimal
+from email.utils import quote
 from functools import wraps
-from secrets import token_hex
 
 from lumibot.brokers import Broker
-from lumibot.entities import Order, Position
+from lumibot.entities import Order, Position, TradingFee
 from lumibot.tools import get_trading_days
 from lumibot.trading_builtins import CustomStream
 
@@ -14,11 +16,6 @@ class BacktestingBroker(Broker):
     # Metainfo
     IS_BACKTESTING_BROKER = True
 
-    CannotPredictFuture = ValueError(
-        "Cannot predict the future. Backtesting datetime already in the present. "
-        "Check if your backtesting end time is set after available data."
-    )
-
     def __init__(self, data_source, connect_stream=True, max_workers=20):
         # Calling init methods
         self.name = "backtesting"
@@ -102,7 +99,7 @@ class BacktestingBroker(Broker):
         now = self.datetime
         search = self._trading_days[now < self._trading_days.market_open]
         if search.empty:
-            raise self.CannotPredictFuture
+            logging.error("Cannot predict future")
 
         return search.market_open[0].to_pydatetime()
 
@@ -112,7 +109,7 @@ class BacktestingBroker(Broker):
 
         search = self._trading_days[now < self._trading_days.market_close]
         if search.empty:
-            raise self.CannotPredictFuture
+            logging.error("Cannot predict future")
 
         trading_day = search.iloc[0]
         if now >= trading_day.market_open:
@@ -123,14 +120,12 @@ class BacktestingBroker(Broker):
 
     # TODO: speed up this function, it is a major bottleneck
     def get_time_to_close(self):
-        import time
-
         """Return the remaining time for the market to close in seconds"""
         now = self.datetime
         # TODO: speed up the next line. next line speed implication: v high (1738 microseconds)
         search = self._trading_days[now < self._trading_days.market_close]
         if search.empty:
-            raise self.CannotPredictFuture
+            logging.error("Cannot predict future")
 
         # TODO: speed up the next line. next line speed implication: high (910 microseconds)
         trading_day = search.iloc[0]
@@ -143,23 +138,33 @@ class BacktestingBroker(Broker):
         delta = trading_day.market_close - now
         return delta.total_seconds()
 
-    def _await_market_to_open(self, timedelta=None):
+    def _await_market_to_open(self, timedelta=None, strategy=None):
         if (
             self._data_source.SOURCE == "PANDAS"
             and self._data_source._timestep == "day"
         ):
             return
+
+        # Process outstanding orders first before waiting for market to open
+        # or else they don't get processed until the next day
+        self.process_pending_orders(strategy=strategy)
+
         time_to_open = self.get_time_to_open()
         if timedelta is not None:
             time_to_open -= 60 * timedelta
         self._update_datetime(time_to_open)
 
-    def _await_market_to_close(self, timedelta=None):
+    def _await_market_to_close(self, timedelta=None, strategy=None):
         if (
             self._data_source.SOURCE == "PANDAS"
             and self._data_source._timestep == "day"
         ):
             return
+
+        # Process outstanding orders first before waiting for market to close
+        # or else they don't get processed until the next day
+        self.process_pending_orders(strategy=strategy)
+
         time_to_close = self.get_time_to_close()
         if timedelta is not None:
             time_to_close -= 60 * timedelta
@@ -179,7 +184,7 @@ class BacktestingBroker(Broker):
         response = Position("", asset, quantity, orders=orders)
         return response
 
-    def _pull_broker_positions(self):
+    def _pull_broker_positions(self, strategy=None):
         """Get the broker representation of all positions"""
         response = self._filled_positions.__items
         return response
@@ -192,7 +197,7 @@ class BacktestingBroker(Broker):
 
     # =======Orders and assets functions=========
 
-    def _parse_broker_order(self, response, strategy):
+    def _parse_broker_order(self, response, strategy_name, strategy_object):
         """parse a broker order representation
         to an order object"""
         order = response
@@ -292,7 +297,7 @@ class BacktestingBroker(Broker):
 
     def submit_order(self, order):
         """Submit an order for an asset"""
-        order.set_identifier(token_hex(16))
+
         order.update_raw(order)
         return order
 
@@ -339,6 +344,35 @@ class BacktestingBroker(Broker):
 
         return orders_closing_contracts
 
+    def calculate_trade_cost(self, order: Order, strategy, price: float):
+        """Calculate the trade cost of an order for a given strategy"""
+        trade_cost = 0
+        trading_fees = []
+        if order.side == "buy":
+            trading_fees: list[TradingFee] = strategy.buy_trading_fees
+        elif order.side == "sell":
+            trading_fees: list[TradingFee] = strategy.sell_trading_fees
+
+        for trading_fee in trading_fees:
+            if trading_fee.taker == True and order.type in [
+                "market",
+                "stop",
+            ]:
+                trade_cost += trading_fee.flat_fee
+                trade_cost += (
+                    Decimal(price) * Decimal(order.quantity) * trading_fee.percent_fee
+                )
+            elif trading_fee.maker == True and order.type in [
+                "limit",
+                "stop_limit",
+            ]:
+                trade_cost += trading_fee.flat_fee
+                trade_cost += (
+                    Decimal(price) * Decimal(order.quantity) * trading_fee.percent_fee
+                )
+
+        return trade_cost
+
     def process_pending_orders(self, strategy):
         """Used to evaluate and execute open orders in backtesting.
 
@@ -353,11 +387,11 @@ class BacktestingBroker(Broker):
         """
 
         # Process expired contracts.
-        pending_orders = self.expired_contracts(strategy)
+        pending_orders = self.expired_contracts(strategy.name)
 
         pending_orders += [
             order
-            for order in self.get_tracked_orders(strategy)
+            for order in self.get_tracked_orders(strategy.name)
             if order.status in ["unprocessed", "new"]
         ]
 
@@ -379,16 +413,16 @@ class BacktestingBroker(Broker):
             filled_quantity = order.quantity
 
             if self._data_source.SOURCE == "YAHOO":
-                ohlc = self.get_last_bar(asset)
-                dt = ohlc.df.index[-1]
-                open = ohlc.df.open[-1]
-                high = ohlc.df.high[-1]
-                low = ohlc.df.low[-1]
-                close = ohlc.df.close[-1]
-                volume = ohlc.df.volume[-1]
+                timeshift = timedelta(
+                    days=-1
+                )  # Is negative so that we get today (normally would get yesterday's data to prevent lookahead bias)
+                ohlc = strategy.get_historical_prices(
+                    asset,
+                    1,
+                    quote=order.quote,
+                    timeshift=timeshift,
+                )
 
-            elif self._data_source.SOURCE == "ALPHA_VANTAGE":
-                ohlc = self.get_last_bar(asset)
                 dt = ohlc.df.index[-1]
                 open = ohlc.df.open[-1]
                 high = ohlc.df.high[-1]
@@ -397,29 +431,38 @@ class BacktestingBroker(Broker):
                 volume = ohlc.df.volume[-1]
 
             elif self._data_source.SOURCE == "PANDAS":
-                data = self._data_source._data_store[asset]
-                ohlc = data._get_bars_dict(self.datetime, length=1)
+                # This is a hack to get around the fact that we need to get the previous day's data to prevent lookahead bias.
+                ohlc = strategy.get_historical_prices(
+                    asset,
+                    1,
+                    quote=order.quote,
+                    timeshift=-1,
+                    timestep=self._data_source._timestep,
+                )
+
                 if ohlc is None:
                     self.cancel_order(order)
                     continue
-                dt = ohlc["datetime"][-1]
-                open = ohlc["open"][-1]
-                high = ohlc["high"][-1]
-                low = ohlc["low"][-1]
-                close = ohlc["close"][-1]
-                volume = ohlc["volume"][-1]
+                dt = ohlc.df.index[-1]
+                open = ohlc.df["open"][-1]
+                high = ohlc.df["high"][-1]
+                low = ohlc.df["low"][-1]
+                close = ohlc.df["close"][-1]
+                volume = ohlc.df["volume"][-1]
 
             # Determine transaction price.
             if order.type == "market":
-                price = open
+                price = close
             elif order.type == "limit":
-                price = self.limit_order(order.limit_price, order.side, open, high, low)
+                price = self.limit_order(
+                    order.limit_price, order.side, close, high, low
+                )
             elif order.type == "stop":
-                price = self.stop_order(order.stop_price, order.side, open, high, low)
+                price = self.stop_order(order.stop_price, order.side, close, high, low)
             elif order.type == "stop_limit":
                 if not order.price_triggered:
                     price = self.stop_order(
-                        order.stop_price, order.side, open, high, low
+                        order.stop_price, order.side, close, high, low
                     )
                     if price != 0:
                         price = self.limit_order(
@@ -428,11 +471,11 @@ class BacktestingBroker(Broker):
                         order.price_triggered = True
                 elif order.price_triggered:
                     price = self.limit_order(
-                        order.limit_price, order.side, open, high, low
+                        order.limit_price, order.side, close, high, low
                     )
             else:
                 raise ValueError(
-                    f"Order type {order.type} is not allowable in backtesting."
+                    f"Order type {order.type} is not implemented for backtesting."
                 )
 
             if price != 0:
@@ -447,6 +490,12 @@ class BacktestingBroker(Broker):
                         )
                         self._new_orders.append(flat_order)
 
+                trade_cost = self.calculate_trade_cost(order, strategy, price)
+
+                new_cash = strategy.cash - float(trade_cost)
+                strategy._set_cash_position(new_cash)
+                order.trade_cost = float(trade_cost)
+
                 self.stream.dispatch(
                     self.FILLED_ORDER,
                     order=order,
@@ -456,52 +505,52 @@ class BacktestingBroker(Broker):
             else:
                 continue
 
-    def limit_order(self, limit_price, side, open, high, low):
+    def limit_order(self, limit_price, side, close, high, low):
         """Limit order logic."""
         if side == "buy":
-            if limit_price >= open:
-                return open
-            elif limit_price < open and limit_price >= low:
+            if limit_price >= close:
+                return close
+            elif limit_price < close and limit_price >= low:
                 return limit_price
             elif limit_price < low:
                 return 0
         elif side == "sell":
-            if limit_price <= open:
-                return open
-            elif limit_price > open and limit_price <= high:
+            if limit_price <= close:
+                return close
+            elif limit_price > close and limit_price <= high:
                 return limit_price
             elif limit_price > high:
                 return 0
 
-    def stop_order(self, stop_price, side, open, high, low):
+    def stop_order(self, stop_price, side, close, high, low):
         """Stop order logic."""
         if side == "buy":
-            if stop_price <= open:
-                return open
-            elif stop_price > open and stop_price <= high:
+            if stop_price <= close:
+                return close
+            elif stop_price > close and stop_price <= high:
                 return stop_price
             elif stop_price > high:
                 return 0
         elif side == "sell":
-            if stop_price >= open:
-                return open
-            elif stop_price < open and stop_price >= low:
+            if stop_price >= close:
+                return close
+            elif stop_price < close and stop_price >= low:
                 return stop_price
             elif stop_price < low:
                 return 0
 
     # =========Market functions=======================
-    def get_last_price(self, asset, quote=None):
+    def get_last_price(self, asset, quote=None, exchange=None, **kwargs):
         """Takes an asset asset and returns the last known price"""
         return self._data_source.get_last_price(asset, quote=quote)
 
-    def get_last_prices(self, symbols, quote=None):
+    def get_last_prices(self, symbols, quote=None, exchange=None, **kwargs):
         """Takes a list of symbols and returns the last known prices"""
         return self._data_source.get_last_prices(symbols, quote=quote)
 
     def get_last_bar(self, asset):
         """Returns OHLCV dictionary for last bar of the asset."""
-        return self._data_source.get_symbol_bars(asset, 1)
+        return self._data_source.get_historical_prices(asset, 1)
 
     def get_chains(self, asset):
         return self._data_source.get_chains(asset)
