@@ -10,10 +10,10 @@ from threading import RLock, Thread
 import pandas as pd
 import pandas_market_calendars as mcal
 from dateutil import tz
-
 from lumibot.data_sources import DataSource
 from lumibot.entities import Order, Position
 from lumibot.trading_builtins import SafeList
+from termcolor import colored
 
 
 class Broker:
@@ -50,9 +50,10 @@ class Broker:
             self._start_orders_thread()
 
         # setting the stream object
-        self.stream = self._get_stream_object()
         if connect_stream:
-            self._launch_stream()
+            self.stream = self._get_stream_object()
+            if self.stream is not None:
+                self._launch_stream()
 
     @property
     def _tracked_orders(self):
@@ -83,7 +84,10 @@ class Broker:
                     flat_orders = self._flatten_order(order)
                     for flat_order in flat_orders:
                         logging.info(
-                            "%r was sent to broker %s" % (flat_order, self.name)
+                            colored(
+                                "%r was sent to broker %s" % (flat_order, self.name),
+                                color="green",
+                            )
                         )
                         self._unprocessed_orders.append(flat_order)
 
@@ -111,11 +115,12 @@ class Broker:
 
     def _set_initial_positions(self, strategy):
         """ Set initial positions """
-        for pos in self._pull_positions(strategy):
+        positions = self._pull_positions(strategy)
+        for pos in positions:
             self._filled_positions.append(pos)
 
     def _process_new_order(self, order):
-        logging.info("New %r was submitted." % order)
+        logging.info(colored("New %r was submitted." % order, color="green"))
         self._unprocessed_orders.remove(order.identifier, key="identifier")
         order.update_status(self.NEW_ORDER)
         order.set_new()
@@ -162,7 +167,10 @@ class Broker:
 
     def _process_filled_order(self, order, price, quantity):
         logging.info(
-            f"Filled Transaction: {order.side} {quantity} of {order.asset.symbol} at {price:,.8f} {'USD'} per share"
+            colored(
+                f"Filled Transaction: {order.side} {quantity} of {order.asset.symbol} at {price:,.8f} {'USD'} per share",
+                color="green",
+            )
         )
         logging.info(f"{order} was filled")
         self._new_orders.remove(order.identifier, key="identifier")
@@ -252,6 +260,9 @@ class Broker:
         check if the limit timestamp was reached"""
         return True
 
+    def market_close_time(self):
+        return self.utc_to_local(self.market_hours(close=True))
+
     def is_market_open(self):
         """Determines if the market is open.
 
@@ -269,15 +280,42 @@ class Broker:
         >>> self.is_market_open()
         True
         """
-        pass
+        open_time = self.utc_to_local(self.market_hours(close=False))
+        close_time = self.utc_to_local(self.market_hours(close=True))
+
+        current_time = datetime.now().astimezone(tz=tz.tzlocal())
+        if self.market == "24/7":
+            return True
+        return (current_time >= open_time) and (close_time >= current_time)
 
     def get_time_to_open(self):
         """Return the remaining time for the market to open in seconds"""
-        pass
+        open_time_this_day = self.utc_to_local(
+            self.market_hours(close=False, next=False)
+        )
+        open_time_next_day = self.utc_to_local(
+            self.market_hours(close=False, next=True)
+        )
+        now = self.utc_to_local(datetime.now())
+        open_time = (
+            open_time_this_day if open_time_this_day > now else open_time_next_day
+        )
+        current_time = datetime.now().astimezone(tz=tz.tzlocal())
+        if self.is_market_open():
+            return 0
+        else:
+            result = open_time.timestamp() - current_time.timestamp()
+            return result
 
     def get_time_to_close(self):
         """Return the remaining time for the market to close in seconds"""
-        pass
+        close_time = self.utc_to_local(self.market_hours(close=True))
+        current_time = datetime.now().astimezone(tz=tz.tzlocal())
+        if self.is_market_open():
+            result = close_time.timestamp() - current_time.timestamp()
+            return result
+        else:
+            return 0
 
     def sleep(self, sleeptime):
         """The broker custom method for sleeping.
@@ -285,7 +323,7 @@ class Broker:
         running live or in backtesting mode"""
         time.sleep(sleeptime)
 
-    def _await_market_to_open(self, timedelta=None):
+    def _await_market_to_open(self, timedelta=None, strategy=None):
         """Executes infinite loop until market opens"""
         isOpen = self.is_market_open()
         if not isOpen:
@@ -297,7 +335,7 @@ class Broker:
             logging.info("Sleeping until the market opens")
             self.sleep(sleeptime)
 
-    def _await_market_to_close(self, timedelta=None):
+    def _await_market_to_close(self, timedelta=None, strategy=None):
         """Sleep until market closes"""
         isOpen = self.is_market_open()
         if isOpen:
@@ -355,15 +393,15 @@ class Broker:
         of the corresponding asset"""
         pass
 
-    def _pull_broker_positions(self):
+    def _pull_broker_positions(self, strategy=None):
         """Get the broker representation of all positions"""
         pass
 
     def _pull_positions(self, strategy):
         """Get the account positions. return a list of
         position objects"""
-        response = self._pull_broker_positions()
-        result = self._parse_broker_positions(response, strategy)
+        response = self._pull_broker_positions(strategy)
+        result = self._parse_broker_positions(response, strategy.name)
         return result
 
     def _pull_position(self, strategy, asset):
@@ -417,18 +455,22 @@ class Broker:
 
         return quantity
 
-    def _parse_broker_order(self, response, strategy):
+    def _parse_broker_order(self, response, strategy_name, strategy_object):
         """parse a broker order representation
         to an order object"""
         pass
 
-    def _parse_broker_orders(self, broker_orders, strategy):
+    def _parse_broker_orders(self, broker_orders, strategy_name, strategy_object):
         """parse a list of broker orders into a
         list of order objects"""
         result = []
         if broker_orders is not None:
             for broker_order in broker_orders:
-                result.append(self._parse_broker_order(broker_order, strategy))
+                result.append(
+                    self._parse_broker_order(
+                        broker_order, strategy_name, strategy_object=strategy_object
+                    )
+                )
         else:
             logging.warning(
                 "No orders found in broker._parse_broker_orders: the broker_orders object is None"
@@ -440,11 +482,11 @@ class Broker:
         """Get a broker order representation by its id"""
         pass
 
-    def _pull_order(self, identifier, strategy):
+    def _pull_order(self, identifier, strategy_name):
         """pull and parse a broker order by id"""
         response = self._pull_broker_order(identifier)
         if response:
-            order = self._parse_broker_order(response, strategy)
+            order = self._parse_broker_order(response, strategy_name)
             return order
         return None
 
@@ -452,11 +494,13 @@ class Broker:
         """Get the broker open orders"""
         pass
 
-    def _pull_open_orders(self, strategy):
+    def _pull_open_orders(self, strategy_name, strategy_object):
         """Get a list of order objects representing the open
         orders"""
         response = self._pull_broker_open_orders()
-        result = self._parse_broker_orders(response, strategy)
+        result = self._parse_broker_orders(
+            response, strategy_name, strategy_object=strategy_object
+        )
         return result
 
     def submit_order(self, order):
@@ -501,11 +545,10 @@ class Broker:
         orders = self.get_tracked_orders(strategy)
         self.cancel_orders(orders)
 
-    def wait_orders_clear(self, strategy, max_loop=8):
+    def wait_orders_clear(self, strategy, max_loop=5):
         # Returns true if outstanding orders for a strategy are complete.
 
-        out_orders = True
-        while out_orders or max_loop > 0:
+        while max_loop > 0:
 
             outstanding_orders = [
                 order
@@ -553,25 +596,18 @@ class Broker:
                 orders.append(order)
         self.submit_orders(orders)
 
-        if not self.IS_BACKTESTING_BROKER:
-            orders_result = self.wait_orders_clear(strategy_name)
-            if not orders_result:
-                logging.info(
-                    "From sell_all, orders were still outstanding after the sell all event"
-                )
-
     # =========Market functions=======================
 
-    def get_last_price(self, asset, quote=None):
+    def get_last_price(self, asset, quote=None, exchange=None, **kwargs):
         """Takes an asset asset and returns the last known price"""
         pass
 
-    def get_last_prices(self, assets, quote=None):
+    def get_last_prices(self, assets, quote=None, exchange=None, **kwargs):
         """Takes a list of assets and returns the last known prices"""
         pass
 
     def _get_tick(self, order):
-        raise NotImplementedError(f"Tic data is not available for {self.name}")
+        raise NotImplementedError(f"Tick data is not available for {self.name}")
 
     # =========Subscribers/Strategies functions==============
 
@@ -738,6 +774,7 @@ class Broker:
                 "price": price,
                 "filled_quantity": filled_quantity,
                 "multiplier": multiplier,
+                "trade_cost": stored_order.trade_cost,
             }
             # append row to the dataframe
             self._trade_event_log_df = pd.concat(
