@@ -1,21 +1,17 @@
 import datetime
+import logging
 import time
-import traceback
 from collections import deque
-from datetime import timezone
 from decimal import Decimal
 from threading import Thread
 
 import pandas as pd
-import pandas_market_calendars as mcal
 from dateutil import tz
 from ibapi.client import *
 from ibapi.contract import *
 from ibapi.order import *
 from ibapi.wrapper import *
-
 from lumibot.data_sources import InteractiveBrokersData
-
 # Naming conflict on Order between IB and Lumibot.
 from lumibot.entities import Asset
 from lumibot.entities import Order as OrderLum
@@ -43,7 +39,19 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
 
         # Connection to interactive brokers
         self.ib = None
-        self.start_ib(config.IP, config.SOCKET_PORT, config.CLIENT_ID)
+
+        # check if the config is a dict
+        if isinstance(config, dict):
+            ip = config["IP"]
+            socket_port = config["SOCKET_PORT"]
+            client_id = config["CLIENT_ID"]
+        else:
+            ip = config.IP
+            socket_port = config.SOCKET_PORT
+            client_id = config.CLIENT_ID
+
+        self.start_ib(ip, socket_port, client_id)
+
         self.market = None
 
     def start_ib(self, ip, socket_port, client_id):
@@ -58,46 +66,6 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
         clock = self.ib.get_timestamp()
         return clock
 
-    def market_close_time(self):
-        return self.utc_to_local(self.market_hours(close=True))
-
-    def is_market_open(self):
-        """Return True if market is open else False"""
-        open_time = self.utc_to_local(self.market_hours(close=False))
-        close_time = self.utc_to_local(self.market_hours(close=True))
-
-        current_time = datetime.datetime.now().astimezone(tz=tz.tzlocal())
-        if self.market == "24/7":
-            return True
-        return (current_time >= open_time) and (close_time >= current_time)
-
-    def get_time_to_open(self):
-        """Return the remaining time for the market to open in seconds"""
-        open_time_this_day = self.utc_to_local(
-            self.market_hours(close=False, next=False)
-        )
-        open_time_next_day = self.utc_to_local(
-            self.market_hours(close=False, next=True)
-        )
-        now = self.utc_to_local(datetime.datetime.now())
-        open_time = (
-            open_time_this_day if open_time_this_day > now else open_time_next_day
-        )
-        current_time = datetime.datetime.now().astimezone(tz=tz.tzlocal())
-        if self.is_market_open():
-            return None
-        else:
-            return open_time.timestamp() - current_time.timestamp()
-
-    def get_time_to_close(self):
-        """Return the remaining time for the market to close in seconds"""
-        close_time = self.utc_to_local(self.market_hours(close=True))
-        current_time = datetime.datetime.now().astimezone(tz=tz.tzlocal())
-        if self.is_market_open():
-            return close_time.timestamp() - current_time.timestamp()
-        else:
-            return None
-
     # =========Positions functions==================
 
     def _parse_broker_position(self, broker_position, strategy, orders=None):
@@ -159,13 +127,22 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
         result = result[result["Symbol"] == asset].squeeze()
         return result
 
-    def _pull_broker_positions(self):
+    def _pull_broker_positions(self, strategy=None):
         """Get the broker representation of all positions"""
-        return [d for d in self.ib.get_positions() if d["position"] > 0]
+        positions = []
+        ib_positions = self.ib.get_positions()
+        if ib_positions:
+            for position in ib_positions:
+                if position["position"] != 0:
+                    positions.append(position)
+        else:
+            logging.info("No positions found at interactive brokers.")
+
+        return positions
 
     # =======Orders and assets functions=========
 
-    def _parse_broker_order(self, response, strategy):
+    def _parse_broker_order(self, response, strategy_name, strategy_object):
         """Parse a broker order representation
         to an order object"""
 
@@ -183,13 +160,13 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
             multiplier = response.contract.multiplier
 
         right = None
-        strike = ""
+        strike = None
         if response.contract.secType == "OPT":
             right = "CALL" if response.contract.right == "C" else "PUT"
             strike = response.contract.strike
 
         order = OrderLum(
-            strategy,
+            strategy_name,
             Asset(
                 symbol=response.contract.localSymbol,
                 asset_type=[
@@ -207,6 +184,7 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
             stop_price=response.auxPrice if response.auxPrice != 0 else None,
             time_in_force=response.tif,
             good_till_date=response.goodTillDate,
+            quote=strategy_object.quote_asset,
         )
         order._transmitted = True
         order.set_identifier(response.orderId)
@@ -261,31 +239,9 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
         order.update_status("submitted")
         return order
 
-    def cancel_order(self, order_id):
+    def cancel_order(self, order):
         """Cancel an order"""
-        self.ib.cancel_order(order_id)
-
-    def cancel_open_orders(self, strategy=None):
-        """Cancel all the strategy open orders"""
-        self.ib.reqGlobalCancel()
-
-    def sell_all(self, strategy_name, cancel_open_orders=True, strategy=None):
-        """Sell all positions"""
-        logging.warning("Strategy %s: sell all" % strategy_name)
-        if cancel_open_orders:
-            self.cancel_open_orders(strategy_name)
-
-        orders = []
-        positions = self.get_tracked_positions(strategy_name)
-
-        for position in positions:
-            size = position.quantity
-            if size == 0:
-                continue
-            side = "sell" if size > 0 else "buy"
-            close_order = OrderLum(strategy_name, position.asset, size, side)
-            orders.append(close_order)
-        self.submit_orders(orders)
+        self.ib.cancel_order(order)
 
     # =========Market functions=======================
 
@@ -319,6 +275,13 @@ class InteractiveBrokers(InteractiveBrokersData, Broker):
         try:
             summary = self.ib.get_account_summary()
         except:
+            logger.error(
+                f"Could not get broker balances. Please check your broker "
+                f"configuration and make sure that TWS is running with the "
+                f"correct configuration. For more information, please "
+                f"see the documentation here: https://lumibot.lumiwealth.com/brokers.interactive_brokers.html"
+            )
+
             return None
         finally:
             if summary is None:
@@ -507,10 +470,11 @@ TYPE_MAP = dict(
     option="OPT",
     future="FUT",
     forex="CASH",
+    index="IND",
 )
 
 DATE_MAP = dict(
-    future="%Y%m",
+    future="%Y%m%d",
     option="%Y%m%d",
 )
 
@@ -586,7 +550,7 @@ class IBWrapper(EWrapper):
 
         # If the last price is not available, then use yesterday's closing price
         # This can happen if the market is closed
-        if tickType == 9 and self.tick is None:
+        if tickType == 9 and self.tick is None and self.should_use_last_close:
             self.tick = price
             self.tick_type_used = tickType
 
@@ -987,10 +951,10 @@ class IBClient(EClient):
         return requested_time
 
     def get_tick(
-        self,
-        asset="",
-        greek=False,
+        self, asset="", greek=False, exchange="SMART", should_use_last_close=True
     ):
+        self.should_use_last_close = should_use_last_close
+
         if not greek:
             tick_storage = self.wrapper.init_tick()
             self.tick_asset = asset
@@ -1000,6 +964,7 @@ class IBClient(EClient):
         contract = self.create_contract(
             asset,
             currency=asset.currency,
+            exchange=exchange,
         )
         reqId = self.get_reqid()
         self.tick_request_id = reqId
@@ -1049,21 +1014,22 @@ class IBClient(EClient):
         parsed_duration="1 D",
         parsed_timestep="1 day",
         type="TRADES",
-        useRTH=1,
+        useRTH=0,
         formatDate=2,
         keepUpToDate=False,
         chartOptions=[],
+        exchange="SMART",
     ):
         historical_storage = self.wrapper.init_historical()
 
-        contract = self.create_contract(symbol)
+        contract = self.create_contract(symbol, exchange=exchange)
         # Call the historical data.
         self.reqHistoricalData(
             self.get_reqid(),
             contract,
             end_date_time,
             parsed_duration,
-            parsed_timestep,
+            parsed_timestep, # barSizeSetting
             type,
             useRTH,
             formatDate,
@@ -1180,11 +1146,12 @@ class IBClient(EClient):
 
         return requested_orders
 
-    def cancel_order(self, order_id):
+    def cancel_order(self, order):
+        order_id = order.identifier
         if not order_id or not isinstance(order_id, int):
             logging.info(
                 f"An attempt to cancel an order without supplying a proper "
-                f"`order_id` was made. This was your `order_id` {order_id}. "
+                f"`order_id` was made. This was your `order_id`: {order_id}. "
                 f"An integer is required. No action was taken."
             )
             return
@@ -1272,7 +1239,7 @@ class IBApp(IBWrapper, IBClient):
     def create_contract(
         self,
         asset,
-        exchange="SMART",
+        exchange=None,
         currency="USD",
         primaryExchange="ISLAND",
     ):
@@ -1281,7 +1248,10 @@ class IBApp(IBWrapper, IBClient):
 
         contract.symbol = str(asset.symbol).upper()
         contract.secType = TYPE_MAP[asset.asset_type]
-        contract.exchange = exchange
+        if exchange is None:
+            contract.exchange = "SMART"
+        else:
+            contract.exchange = exchange
         contract.currency = currency
 
         if asset.asset_type == "stock":
@@ -1293,11 +1263,14 @@ class IBApp(IBWrapper, IBClient):
             contract.multiplier = asset.multiplier
             contract.primaryExchange = "CBOE"
         elif asset.asset_type == "future":
-            contract.lastTradeDateOrContractMonth = asset.expiration.strftime("%Y%m")
-            contract.primaryExchange = "GLOBEX"
-            contract.currency = "USD"
+            if exchange is None:
+                contract.exchange = "CME"
+            contract.includeExpired = True
+            contract.lastTradeDateOrContractMonth = asset.expiration.strftime("%Y%m%d")
         elif asset.asset_type == "forex":
             contract.exchange = "IDEALPRO"
+        elif asset.asset_type == "index":
+            pass
         else:
             raise ValueError(
                 f"The asset {asset.symbol} has a type of {asset.asset_type}. "
@@ -1315,7 +1288,7 @@ class IBApp(IBWrapper, IBClient):
                     f"order. The bracket order for {order.symbol} is cancelled."
                 )
                 return []
-            parent = Order()
+            parent = Order()  
             parent.orderId = (
                 order.identifier if order.identifier else self.nextOrderId()
             )
@@ -1326,7 +1299,7 @@ class IBApp(IBWrapper, IBClient):
             parent.transmit = False
 
             takeProfit = Order()
-            takeProfit.orderId = parent.orderId + 1
+            takeProfit.orderId = self.nextOrderId() 
             takeProfit.action = "SELL" if parent.action == "BUY" else "BUY"
             takeProfit.orderType = "LMT"
             takeProfit.totalQuantity = order.quantity
@@ -1335,7 +1308,7 @@ class IBApp(IBWrapper, IBClient):
             takeProfit.transmit = False
 
             stopLoss = Order()
-            stopLoss.orderId = parent.orderId + 2
+            stopLoss.orderId = self.nextOrderId() 
             stopLoss.action = "SELL" if parent.action == "BUY" else "BUY"
             stopLoss.orderType = "STP"
             stopLoss.auxPrice = order.stop_loss_price
@@ -1367,7 +1340,7 @@ class IBApp(IBWrapper, IBClient):
 
             if order.take_profit_price:
                 takeProfit = Order()
-                takeProfit.orderId = parent.orderId + 1
+                takeProfit.orderId = self.nextOrderId()
                 takeProfit.action = "SELL" if parent.action == "BUY" else "BUY"
                 takeProfit.orderType = "LMT"
                 takeProfit.totalQuantity = order.quantity
@@ -1378,7 +1351,7 @@ class IBApp(IBWrapper, IBClient):
 
             elif order.stop_loss_price:
                 stopLoss = Order()
-                stopLoss.orderId = parent.orderId + 1
+                stopLoss.orderId = self.nextOrderId()
                 stopLoss.action = "SELL" if parent.action == "BUY" else "BUY"
                 stopLoss.orderType = "STP"
                 stopLoss.auxPrice = order.stop_loss_price
@@ -1403,7 +1376,7 @@ class IBApp(IBWrapper, IBClient):
             takeProfit.ocaType = 1
 
             stopLoss = Order()
-            stopLoss.orderId = takeProfit.orderId + 1
+            stopLoss.orderId = self.nextOrderId()
             stopLoss.action = order.side.upper()
             stopLoss.orderType = "STP"
             stopLoss.totalQuantity = order.quantity
