@@ -1,6 +1,7 @@
 import logging
 from collections import namedtuple
 from decimal import Decimal
+from secrets import token_hex
 from threading import Event
 
 import lumibot.entities as entities
@@ -29,12 +30,14 @@ class Order:
         time_in_force="day",
         good_till_date=None,
         sec_type=None,
-        exchange="SMART",
+        exchange=None,
         position_filled=False,
         quote=None,
         pair=None,
         date_created=None,
         type=None,
+        trade_cost: float = None,
+        custom_params={},
     ):
         """Order class for managing individual orders.
 
@@ -112,7 +115,7 @@ class Order:
             The order has been filled.
         exchange : str
             The exchange where the order will be placed.
-            Default = `SMART`
+            Default = None
         quote : Asset
             This is the base cryptocurrency. For example, if trading
             `BTC/ETH` this parameter will be 'ETH' (as an Asset object).
@@ -122,6 +125,11 @@ class Order:
             The date the order was created.
         type : str
             The type of order. Possible values are: `market`, `limit`, `stop`, `stop_limit`, `trail`, `trail_limit`, `bracket`, `bracket_limit`, `bracket_stop`, `bracket_stop_limit`.
+        trade_cost : float
+            The cost of this order in the quote currency.
+        custom_params : dict
+            A dictionary of custom parameters that can be used to pass additional information to the broker. This is useful for passing custom parameters to the broker that are not supported by Lumibot.
+            Eg. `custom_params={"leverage": 3}` for Kraken margin trading.
         Examples
         --------
         >>> from lumibot.entities import Asset
@@ -191,9 +199,9 @@ class Order:
             self.quote = quote
 
         self.symbol = self.asset.symbol
-        self.identifier = None
+        self.identifier = token_hex(16)
         self.status = "unprocessed"
-        self._date_created = None
+        self._date_created = date_created
         self.side = None
         self.time_in_force = time_in_force
         self.good_till_date = good_till_date
@@ -211,6 +219,8 @@ class Order:
         self.dependent_order = None
         self.dependent_order_filled = False
         self.type = type
+        self.trade_cost = trade_cost
+        self.custom_params = custom_params
 
         # Options:
         self.exchange = exchange
@@ -357,11 +367,6 @@ class Order:
         if not isinstance(value, Decimal):
             if isinstance(value, float):
                 value = Decimal(str(value))
-            else:
-                try:
-                    assert isinstance(value, int), error_msg
-                except TypeError as error:
-                    logging.info(error)
 
         quantity = Decimal(value)
         self._quantity = check_quantity(
@@ -379,12 +384,7 @@ class Order:
                 f"{self.symbol} {self.asset.expiration} "
                 f"{self.asset.right} {self.asset.strike}"
             )
-        repr = "%s order of | %f %s %s |" % (
-            self.type,
-            self.quantity,
-            self.rep_asset,
-            self.side,
-        )
+        repr = f"{self.type} order of | {self.quantity} {self.rep_asset} {self.side} |"
         if self.order_class:
             repr = "%s of class %s" % (repr, self.order_class)
         repr = "%s with status %s" % (repr, self.status)
