@@ -8,19 +8,12 @@ from decimal import Decimal
 import jsonpickle
 import pandas as pd
 from attr import has
-
 from lumibot import LUMIBOT_DEFAULT_PYTZ
 from lumibot.backtesting import BacktestingBroker
-from lumibot.entities import Asset, Position
-from lumibot.tools import (
-    create_tearsheet,
-    day_deduplicate,
-    get_risk_free_rate,
-    get_symbol_returns,
-    plot_returns,
-    stats_summary,
-    to_datetime_aware,
-)
+from lumibot.entities import Asset, Position, TradingFee
+from lumibot.tools import (create_tearsheet, day_deduplicate,
+                           get_risk_free_rate, get_symbol_returns,
+                           plot_returns, stats_summary, to_datetime_aware)
 from lumibot.traders import Trader
 
 from .strategy_executor import StrategyExecutor
@@ -49,6 +42,8 @@ class _Strategy:
         name=None,
         budget=None,
         parameters={},
+        buy_trading_fees=[],
+        sell_trading_fees=[],
         **kwargs,
     ):
         # Handling positional arguments.
@@ -60,6 +55,9 @@ class _Strategy:
 
         # TODO: Break up this function, too long!
 
+        self.buy_trading_fees = buy_trading_fees
+        self.sell_trading_fees = sell_trading_fees
+
         if len(args) == 1:
             if isinstance(args[0], str):
                 self._name = args[0]
@@ -109,7 +107,7 @@ class _Strategy:
                     assert pandas_data != None
                 except AssertionError:
                     raise ValueError(
-                        f"Please add a pandas dataframe as an input parameter. "
+                        f"Pandas data is required when using pandas for backtesting. Please add a pandas dataframe as an input parameter. "
                         f"Use the following: 'pandas_data': your_dataframe "
                     )
                 self.broker._trading_days = self.data_source.load_data()
@@ -143,13 +141,10 @@ class _Strategy:
         self._first_iteration = True
         self._last_on_trading_iteration_datetime = None
         if not self._is_backtesting:
-            (
-                self._cash,
-                self._position_value,
-                self._portfolio_value,
-            ) = self.broker._get_balances_at_broker(self.quote_asset)
+            self.update_broker_balances()
+
             # Set initial positions if live trading.
-            self.broker._set_initial_positions(self._name)
+            self.broker._set_initial_positions(self)
         else:
             if budget is None:
                 if self.cash is None:
@@ -164,6 +159,9 @@ class _Strategy:
                 )
                 self._set_cash_position(budget)
 
+            ##############################################
+            ### TODO: Should all this just use _update_portfolio_value()?
+            ### START
             self._portfolio_value = self.cash
 
             store_assets = list(self.broker._data_source._data_store.keys())
@@ -186,6 +184,9 @@ class _Strategy:
             else:
                 self._position_value = 0
 
+            ### END
+            ##############################################
+
         self._initial_budget = budget
         self._minutes_before_closing = minutes_before_closing
         self._minutes_before_opening = minutes_before_opening
@@ -195,14 +196,20 @@ class _Strategy:
 
         # Stats related variables
         self._stats_file = stats_file
-        self._stats = pd.DataFrame()
+        self._stats = None
+        self._stats_list = []
         self._analysis = {}
 
         # Storing parameters for the initialize method
-        if not hasattr(self, "parameters") or type(self.parameters) != dict:
+        if (
+            not hasattr(self, "parameters")
+            or type(self.parameters) != dict
+            or self.parameters is None
+        ):
             self.parameters = {}
         self.parameters = {**self.parameters, **kwargs}
-        self.parameters = {**self.parameters, **parameters}
+        if parameters is not None and type(self.parameters) == dict:
+            self.parameters = {**self.parameters, **parameters}
 
         self._strategy_returns_df = None
         self._benchmark_returns_df = None
@@ -216,7 +223,7 @@ class _Strategy:
         for key in self.__dict__:
             if key[0] != "_" and key not in ignored_fields:
                 try:
-                    result[key] = deepcopy(self.__dict__[key])
+                    result[key] = self.__dict__[key]
                 except:
                     pass
                     # logging.warning(
@@ -232,7 +239,7 @@ class _Strategy:
                 "_sleeptime",
                 "_is_backtesting",
             ]:
-                result[key[1:]] = deepcopy(self.__dict__[key])
+                result[key[1:]] = self.__dict__[key]
 
         return result
 
@@ -296,12 +303,70 @@ class _Strategy:
         """Returns the name of the strategy as a string if not default"""
         return f"{self._name} " if self._name != None else ""
 
+    def update_broker_balances(self, force_update=True):
+        """Updates the broker's balances, including cash and portfolio value
+
+        Parameters
+        ----------
+        force_update : bool, optional
+            If True, forces the broker to update the balances immediately.
+            If False, the broker will only update the balances if the last
+            update was more than 1 minute ago. The default is True.
+
+        Returns
+        -------
+        bool
+            True if the broker's balances were updated, False otherwise
+        """
+        if self._is_backtesting:
+            return True
+
+        if "last_broker_balances_update" not in self.__dict__:
+            self.last_broker_balances_update = None
+
+        UPDATE_INTERVAL = 59
+        if (
+            self.last_broker_balances_update is None
+            or force_update
+            or (
+                self.last_broker_balances_update
+                + datetime.timedelta(seconds=UPDATE_INTERVAL)
+                < datetime.datetime.now()
+            )
+        ):
+
+            broker_balances = self.broker._get_balances_at_broker(self.quote_asset)
+
+            if broker_balances is not None:
+                (
+                    self._cash,
+                    self._position_value,
+                    self._portfolio_value,
+                ) = broker_balances
+
+                self.last_broker_balances_update = datetime.datetime.now()
+                return True
+
+            else:
+                logger.error(
+                    "Unable to get balances (cash, portfolio value, etc) from broker. "
+                    "Please check your broker and your broker configuration."
+                )
+                return False
+        else:
+            logger.debug("Balances already updated recently. Skipping update.")
+
     # =============Auto updating functions=============
 
     def _update_portfolio_value(self):
         """updates self.portfolio_value"""
         if not self._is_backtesting:
-            return self.broker._get_balances_at_broker(self.quote_asset)[2]
+            broker_balances = self.broker._get_balances_at_broker(self.quote_asset)
+
+            if broker_balances is not None:
+                return broker_balances[2]
+            else:
+                return None
 
         with self._executor.lock:
             # Used for traditional brokers, for crypto this could be 0
@@ -423,10 +488,12 @@ class _Strategy:
     # =============Stats functions=====================
 
     def _append_row(self, row):
-        self._stats = pd.concat([self._stats, pd.DataFrame(row, index=[0])])
+        self._stats_list.append(row)
 
     def _format_stats(self):
-        self._stats = self._stats.set_index("datetime")
+        self._stats = pd.DataFrame(self._stats_list)
+        if "datetime" in self._stats.columns:
+            self._stats = self._stats.set_index("datetime")
         self._stats["return"] = self._stats["portfolio_value"].pct_change()
         return self._stats
 
@@ -438,7 +505,7 @@ class _Strategy:
                 current_stream_handler_level = handler.level
                 handler.setLevel(logging.INFO)
         logger.setLevel(logging.INFO)
-        if not self._stats.empty:
+        if len(self._stats_list) > 0:
             self._format_stats()
             if self._stats_file:
                 self._stats.to_csv(self._stats_file)
@@ -611,6 +678,8 @@ class _Strategy:
         save_tearsheet=True,
         show_tearsheet=True,
         parameters={},
+        buy_trading_fees=[],
+        sell_trading_fees=[],
         **kwargs,
     ):
         """Backtest a strategy.
@@ -649,8 +718,9 @@ class _Strategy:
             The file to write the plot html to.
         trades_file : str
             The file to write the trades to.
-        pandas_data : pandas.DataFrame
-            The pandas data to use.
+        pandas_data : list
+            A list of Data objects that are used when the datasource_class object is set to PandasDataBacktesting.
+            This contains all the data that will be used in backtesting.
         quote_asset : Asset (crypto)
             An Asset object for the crypto currency that will get used
             as a valuation asset for measuring overall porfolio values.
@@ -668,6 +738,10 @@ class _Strategy:
         parameters : dict
             A dictionary of parameters to pass to the strategy. These parameters
             must be set up within the initialize() method.
+        buy_trading_fees : list of TradingFee objects
+            A list of TradingFee objects to apply to the buy orders during backtests.
+        sell_trading_fees : list of TradingFee objects
+            A list of TradingFee objects to apply to the sell orders during backtests.
 
         Returns
         -------
@@ -745,6 +819,15 @@ class _Strategy:
                 "for your three positional arguments. \n"
             )
 
+        if datasource_class.SOURCE == "PANDAS":
+            try:
+                assert pandas_data != None and len(pandas_data) > 0
+            except AssertionError:
+                raise ValueError(
+                    f"Pandas data is required when using pandas for backtesting. Please add a pandas dataframe as an input parameter. "
+                    f"Use the following: 'pandas_data': your_dataframe "
+                )
+
         if name is None:
             name = cls.__name__
 
@@ -752,7 +835,7 @@ class _Strategy:
         datestring = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
         if plot_file_html is None:
             plot_file_html = (
-                f"logs/{name + '_' if name != None else ''}{datestring}.html"
+                f"logs/{name + '_' if name != None else ''}{datestring}_trades.html"
             )
         if stats_file is None:
             stats_file = (
@@ -818,6 +901,8 @@ class _Strategy:
             name=name,
             budget=budget,
             parameters=parameters,
+            buy_trading_fees=buy_trading_fees,
+            sell_trading_fees=sell_trading_fees,
             **kwargs,
         )
         trader.add_strategy(strategy)
