@@ -8,6 +8,7 @@ from queue import Empty, Queue
 from threading import Event, Lock, Thread
 
 from lumibot.tools import append_locals, lumibot_time, staticdecorator
+from termcolor import colored
 
 
 class StrategyExecutor(Thread):
@@ -109,8 +110,8 @@ class StrategyExecutor(Thread):
                 if cash_broker is not None:
                     self.strategy._set_cash_position(cash_broker)
 
-                positions_broker = self.broker._pull_positions(self.name)
-                orders_broker = self.broker._pull_open_orders(self.name)
+                positions_broker = self.broker._pull_positions(self.strategy)
+                orders_broker = self.broker._pull_open_orders(self.name, self.strategy)
 
                 held_trades_len = len(self.broker._held_trades)
                 if held_trades_len > 0:
@@ -184,7 +185,7 @@ class StrategyExecutor(Thread):
                                 if olumi != obroker:
                                     setattr(order_lumi, order_attr, obroker)
                                     logging.warning(
-                                        f"We would adjust {order_lumi}, {order_attr}, to be {obroker} her."
+                                        f"We are adjusting the {order_attr} of the order {order_lumi}, from {olumi} to be {obroker} because what we have in memory does not match the broker."
                                     )
                         else:
                             # Add to order in lumibot.
@@ -366,7 +367,8 @@ class StrategyExecutor(Thread):
         when an exception is raised and the bot crashes"""
         self.strategy.log_message("Executing the on_bot_crash event method")
         self.strategy.on_bot_crash(error)
-        self.strategy._dump_stats()
+        if self.broker.IS_BACKTESTING_BROKER:
+            self.strategy._dump_stats()
 
     def _on_abrupt_closing(self, error):
         """Use this lifecycle event to execute code
@@ -440,16 +442,16 @@ class StrategyExecutor(Thread):
         else:
             raise ValueError(sleeptime_err_msg)
 
-        if units not in "SMHD":
+        if units not in "SMHDsmhd":
             raise ValueError(sleeptime_err_msg)
 
-        if units == "S":
+        if units == "S" or units == "s":
             strategy_sleeptime = time
-        elif units == "M":
+        elif units == "M" or units == "m":
             strategy_sleeptime = 60 * time
-        elif units == "H":
+        elif units == "H" or units == "h":
             strategy_sleeptime = 60 * 60 * time
-        elif units == "D":
+        elif units == "D" or units == "d":
             strategy_sleeptime = 60 * 60 * 24 * time
         else:
             strategy_sleeptime = time
@@ -461,7 +463,9 @@ class StrategyExecutor(Thread):
         ):
             return False
         else:
-            self.strategy.log_message(f"Sleeping for {strategy_sleeptime} seconds")
+            self.strategy.log_message(
+                colored(f"Sleeping for {strategy_sleeptime} seconds", color="blue")
+            )
             # TODO: next line speed implication: medium (371 microseconds)
             self.safe_sleep(strategy_sleeptime)
 
@@ -495,24 +499,23 @@ class StrategyExecutor(Thread):
             ]
 
             self.broker._update_datetime(dt)
-            # Is this update money dividends in the right place? Maybe after orders. or both
-            if self.broker.IS_BACKTESTING_BROKER:
-                self.broker.process_pending_orders(strategy=self.strategy.name)
+
             self.strategy._update_cash_with_dividends()
             self._on_trading_iteration()
+            if self.broker.IS_BACKTESTING_BROKER:
+                self.broker.process_pending_orders(strategy=self.strategy)
             return
 
-        if not is_247 and (
-            not has_data_source
-            or (has_data_source and self.broker._data_source.SOURCE != "PANDAS")
-        ):
-            self.strategy.await_market_to_open()  # set new time and bar length. Check if hit bar max
-            # or date max.
-            if not self.broker.is_market_open():
-                self._before_market_opens()
-            self.strategy._update_cash_with_dividends()
-
         if not is_247:
+            if not has_data_source or (
+                has_data_source and self.broker._data_source.SOURCE != "PANDAS"
+            ):
+                self.strategy.await_market_to_open()  # set new time and bar length. Check if hit bar max
+                # or date max.
+                if not self.broker.is_market_open():
+                    self._before_market_opens()
+                self.strategy._update_cash_with_dividends()
+
             self.strategy.await_market_to_open(timedelta=0)
             self._before_starting_trading()
 
@@ -531,12 +534,12 @@ class StrategyExecutor(Thread):
             ):
                 break
 
-            if self.broker.IS_BACKTESTING_BROKER:
-                self.broker.process_pending_orders(strategy=self.strategy.name)
-
             # TODO: next line speed implication: v high (7563 microseconds) _on_trading_iteration()
             self._on_trading_iteration()
 
+            if self.broker.IS_BACKTESTING_BROKER:
+                self.broker.process_pending_orders(strategy=self.strategy)
+
             # Sleep until the next trading iteration
             # TODO: next line speed implication: high (2625 microseconds) _strategy_sleep()
             if not self._strategy_sleep():
@@ -544,8 +547,7 @@ class StrategyExecutor(Thread):
 
         self.strategy.await_market_to_close()
         if self.broker.is_market_open():
-            self._before_market_closes()  # perhaps the user could set the time of day based on
-            # their data that the market closes?
+            self._before_market_closes()  # perhaps the user could set the time of day based on their data that the market closes?
 
         self.strategy.await_market_to_close(timedelta=0)
         self._after_market_closes()
@@ -566,7 +568,11 @@ class StrategyExecutor(Thread):
                 # The bot crashed so log the error, call the on_bot_crash method, and continue
                 logging.error(e)
                 logging.error(traceback.format_exc())
-                self._on_bot_crash(e)
+                try:
+                    self._on_bot_crash(e)
+                except Exception as e:
+                    logging.error(e)
+                    logging.error(traceback.format_exc())
 
                 # Only stop the strategy if it's time, otherwise keep running the bot
                 if not self._strategy_sleep():
